//--------------------------------------------------------------
// File     : stm32_ub_max5250.c
// Datum    : 04.03.2013
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F4
// IDE      : CooCox CoIDE 1.7.0
// Module   : GPIO, STM32_UB_SPI2
// Funktion : MAX5250 (4fach 10bit DAC)
//
// Hinweis  : Settings :
//            SPI-Mode = 0, FRQ-Max = 10MHz
//            SPI2 [CLK=PB13, MOSI=PB15, MISO=PB14]
//            Chip-Select an PB12
//--------------------------------------------------------------

//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_max5250.h"
#include "stm32_ub_spi2.h"



//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void P_MAX5250_initIO(void);
void P_MAX5250_CS(BitAction wert);
void P_MAX5250_SendFrame(uint16_t frame);



//--------------------------------------------------------------
// Initialisierung vom MAX5250
// Return_wert :
//  -> ERROR   , wenn Initialisierung fehlgeschlagen
//  -> SUCCESS , wenn Initialisierung ok war 
//--------------------------------------------------------------
ErrorStatus UB_MAX5250_Init(void)
{
  ErrorStatus ret_wert=ERROR;
  // Init der Chip-Select-Leitung
  P_MAX5250_initIO();
  // ChipSelect auf Hi
  P_MAX5250_CS(Bit_SET);
  // init vom SPI-2 im Mode-0
  ret_wert=UB_SPI2_Init(SPI_MODE_0);  

  return(ret_wert);
}


//--------------------------------------------------------------
// Stellt einen DAC-Wert ein
// Kanal : [DAC_0, DAC_1, DAC_2, DAC_3]
// Wert  : 0 bis 1023
//--------------------------------------------------------------
void UB_MAX5250_SetDAC(MAX5250_DAC_t kanal, uint16_t wert)
{
  uint16_t frame=0;

  // Maxwert = 10bit
  if(wert>1023) wert=1023;

  // Frame zusammenstellen
  frame=(wert<<2);
  if(kanal==DAC_0) frame|=0x3000; // DAC0 setzen
  if(kanal==DAC_1) frame|=0x7000; // DAC1 setzen
  if(kanal==DAC_2) frame|=0xB000; // DAC2 setzen
  if(kanal==DAC_3) frame|=0xF000; // DAC3 setzen   

  // Frame senden
  P_MAX5250_SendFrame(frame);  
}


//--------------------------------------------------------------
// interne Funktion
// Init der ChipSelect-Leitung
//--------------------------------------------------------------
void P_MAX5250_initIO(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  // Init der Chip-Select-Leitung
  RCC_AHB1PeriphClockCmd(MAX5250_CS_GPIO_CLK,ENABLE);

  GPIO_InitStructure.GPIO_Pin = MAX5250_CS_PIN;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(MAX5250_CS_GPIO_PORT, &GPIO_InitStructure);
 
}


//--------------------------------------------------------------
// interne Funktion
// Pegel von ChipSelect einstellen
//--------------------------------------------------------------
void P_MAX5250_CS(BitAction wert) {
  if(wert==Bit_RESET) {
    MAX5250_CS_GPIO_PORT->BSRRH = MAX5250_CS_PIN;
  }
  else {
    MAX5250_CS_GPIO_PORT->BSRRL = MAX5250_CS_PIN;
  }
}


//--------------------------------------------------------------
// interne Funktion
// sendet einen Frame (16bit) per SPI
//--------------------------------------------------------------
void P_MAX5250_SendFrame(uint16_t frame)
{
  uint8_t wert;

  // ChipSelect auf Lo
  P_MAX5250_CS(Bit_RESET);

  // hi_byte senden
  wert=(uint8_t)(frame>>8);
  UB_SPI2_SendByte(wert);

  // lo_byte senden
  wert=(uint8_t)(frame&0x00FF);
  UB_SPI2_SendByte(wert);

  // ChipSelect auf Hi
  P_MAX5250_CS(Bit_SET);  
}
